Bonjour,

Un administrateur à lancer la procédure de reset de mot de passe. Ton ancien mot de passe ne fonctionne plus.
Voici un mot de passe temporaire que tu devras changer à ta prochaine connexion :

**lien** => `https://accounts.google.com/ServiceLogin`
**email** => `{{ email }}`
**mot de passe temporaire** => `{{ password|safe }}`

Pense bien à sauvegarder ton nouveau mot de passe dans un outil comme Keepass pour éviter de l'oublier !
