Bonjour à toi !

Voici les informations pour te connecter :

**lien** => `https://accounts.google.com/ServiceLogin`
**email** => `{{ email }}`
**mot de passe** => `{{ password|safe }}`

Tu seras invité à changer ton mot de passe lors de la première connexion.

Tu as accès maintenant aux divers outils de l'asso, n'hésite pas à naviguer avec ton adresse mail Lyon e-Sport :
- Drive (gestionnaire de fichiers) : https://drive.google.com/drive/shared-drives
- Wiki (base de connaissance) : https://wiki.lyon-esport.fr/
(attention à bien utiliser à la première connexion, il faut cliquer sur "Vous n'avez pas de compte ?" puis "S'inscrire avec Google")

Avec ces liens tu verras  l'essentiel du travail de l'asso. Bien évidemment, tout ce qui concerne la LES sera uniquement ici.

Le compte Google LES te permettra d'utiliser l'ensemble des services liés aux GAPPS/Workspace !
Si tu as des questions, n'hésite surtout pas à revenir vers les différents chan de la #Technique ! On se fera un plaisir de te répondre.

Bienvenue à la Lyon e-Sport ! :wink: :grin:
