<div id="sig-container">
	<table cellpadding="0" cellspacing="0">
		<tr>
			<div style="float: left; margin: 2px 5px 5px 0px; display: block;" id="photoWrapper">
				<td>
					<span><a href="https://www.lyon-esport.fr/"><img src="https://admin.lyon-esport.fr/media/uploads/images/logo_les.png" id="sigPhoto" width="124px" height="111px" alt="Logo et site Lyon e-Sport"></a></span>&nbsp;
				</td>
			</div>
			<td color="#F2547D" direction="vertical" class="sc-jhAzac hmXDXQ" style="width: 2px; margin-right: 5px; margin-left: 5px; border-bottom: none; border-right: 1px solid rgb(42, 42, 46);" width="1"></td>
			<div style="margin-top:0px; margin-right: 5px;" id="sigDetailsWrapper">
				<td width="342px">
					<table width="370" cellspacing="3" cellpadding="3" border="0">
						<tr width="342px" height="25px">
							<p style="font-family: Helvetica, sans-serif; font-size: 12px; line-height: 14px; color:#333; margin-top: 0px;" id="sigCompanyWrapper">
								<strong><span id="sigName" style="color:rgb(255, 0, 0);">{{ firstname|title }} {{ lastname|upper }}</span></strong>
								{% if role is not none %}<br><span id="sigTitle">{{ role }}</span>{% endif %}
								{% if team is not none %}<br><span id="sigTeam">Equipe {{ team }}</span>{% endif %}
								<br><strong><span id="sigCompany">Association Lyon e-Sport</span></strong>
							</p>
						</tr>
						<tr width="342px" height="25px">
							<span id="twitterIcon"><a href="https://twitter.com/LyoneSport"><img src="https://admin.lyon-esport.fr/media/uploads/images/twitter-social-logotype.png" width="20px" height="20px" alt="Twitter Lyon e-Sport"></a></span>
							&nbsp;<span id="instagramIcon"><a href="https://www.instagram.com/lyonesport/"><img src="https://admin.lyon-esport.fr/media/uploads/images/instagram-social-network-logo-of-photo-camera.png" width="20px" height="20px" alt="Instagram Lyon e-Sport"></a></span>
							&nbsp;<span id="youtubeIcon"><a href="https://www.youtube.com/c/lyonesports/"><img src="https://admin.lyon-esport.fr/media/uploads/images/youtube-logotype.png" width="20px" height="20px" alt="YouTube Lyon e-Sport"></a></span>
							{% if team == "technique" %}&nbsp;<span id="githubIcon"><a href="https://github.com/lyon-esport/"><img src="https://admin.lyon-esport.fr/media/uploads/images/github.png" width="20px" height="20px" alt="GitHub Lyon e-Sport"></a></span>{% endif %}
							&nbsp;<span id="websiteIcon"><a href="https://www.lyon-esport.fr/"><img src="https://admin.lyon-esport.fr/media/uploads/images/world-wide-web.png" width="20px" height="20px" alt="Site Lyon e-Sport"></a></span>
						</tr>
					</table>
				</td>
			</div>
		</tr>
	</table>
</div>
